package com.example.refactor.aspect;

import com.example.refactor.annotation.SnapshotAware;
import com.example.refactor.entity.User;
import com.example.refactor.service.SnapshotService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 快照切面
 * 使用AOP自动处理快照逻辑
 */
@Aspect
@Component
public class SnapshotAspect {
    
    @Autowired(required = false)
    private SnapshotService snapshotService;
    
    /**
     * 拦截所有标记了@SnapshotAware的方法
     */
    @Around("@annotation(snapshotAware)")
    public Object aroundSnapshotAware(ProceedingJoinPoint joinPoint, SnapshotAware snapshotAware) throws Throwable {
        if (snapshotService == null) {
            // 如果快照服务未启用，直接执行原方法
            return joinPoint.proceed();
        }
        
        Object[] args = joinPoint.getArgs();
        
        switch (snapshotAware.operation()) {
            case QUERY:
                return handleQuery(joinPoint, args);
            case UPDATE:
                return handleUpdate(joinPoint, args, snapshotAware);
            case DELETE:
                return handleDelete(joinPoint, args, snapshotAware);
            case CREATE:
                return handleCreate(joinPoint, args);
            default:
                return joinPoint.proceed();
        }
    }
    
    /**
     * 处理查询操作
     */
    private Object handleQuery(ProceedingJoinPoint joinPoint, Object[] args) throws Throwable {
        Object result = joinPoint.proceed();
        
        if (result instanceof User) {
            // 单个用户查询
            User user = (User) result;
            if (user != null) {
                return snapshotService.getUserWithSnapshot(user.getId());
            }
        } else if (result instanceof List) {
            // 用户列表查询
            List<?> users = (List<?>) result;
            if (!users.isEmpty() && users.get(0) instanceof User) {
                return users.stream()
                        .map(user -> snapshotService.getUserWithSnapshot(((User) user).getId()))
                        .filter(user -> user != null)
                        .collect(Collectors.toList());
            }
        }
        
        return result;
    }
    
    /**
     * 处理更新操作
     */
    private Object handleUpdate(ProceedingJoinPoint joinPoint, Object[] args, SnapshotAware snapshotAware) throws Throwable {
        // 在更新前创建快照
        if (snapshotAware.createSnapshotBefore() && args.length > 0 && args[0] instanceof Long) {
            Long userId = (Long) args[0];
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                String reason = snapshotAware.snapshotReason();
                if (reason.isEmpty()) {
                    reason = "用户信息更新";
                }
                snapshotService.createSnapshot(userId, reason);
            }
        }
        
        return joinPoint.proceed();
    }
    
    /**
     * 处理删除操作
     */
    private Object handleDelete(ProceedingJoinPoint joinPoint, Object[] args, SnapshotAware snapshotAware) throws Throwable {
        // 在删除前创建快照
        if (snapshotAware.createSnapshotBefore() && args.length > 0 && args[0] instanceof Long) {
            Long userId = (Long) args[0];
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                String reason = snapshotAware.snapshotReason();
                if (reason.isEmpty()) {
                    reason = "用户删除前备份";
                }
                snapshotService.createSnapshot(userId, reason);
            }
        }
        
        return joinPoint.proceed();
    }
    
    /**
     * 处理创建操作
     */
    private Object handleCreate(ProceedingJoinPoint joinPoint, Object[] args) throws Throwable {
        // 创建操作不需要快照处理
        return joinPoint.proceed();
    }
} 