package com.example.refactor.aspect;

import com.example.refactor.entity.User;
import com.example.refactor.service.SnapshotService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户实体切面
 * 通过方法签名模式自动拦截所有操作User实体的方法
 * 无需手动添加注解，完全自动化
 */
@Aspect
@Component
public class UserEntityAspect {
    
    @Autowired(required = false)
    private SnapshotService snapshotService;
    
    /**
     * 拦截所有返回User的方法（查询操作）
     * 匹配模式：* com.example.refactor.service.*Service.*(Long)
     */
    @Around("execution(* com.example.refactor.service.*Service.getUser*(Long)) || " +
            "execution(* com.example.refactor.service.*Service.findUser*(Long)) || " +
            "execution(* com.example.refactor.service.*Service.queryUser*(Long))")
    public Object aroundUserQuery(ProceedingJoinPoint joinPoint) throws Throwable {
        if (snapshotService == null) {
            return joinPoint.proceed();
        }
        
        Object result = joinPoint.proceed();
        if (result instanceof User) {
            User user = (User) result;
            if (user != null) {
                return snapshotService.getUserWithSnapshot(user.getId());
            }
        }
        return result;
    }
    
    /**
     * 拦截所有返回List<User>的方法（批量查询操作）
     * 匹配模式：* com.example.refactor.service.*Service.get*Users*()
     */
    @Around("execution(* com.example.refactor.service.*Service.get*Users*()) || " +
            "execution(* com.example.refactor.service.*Service.find*Users*()) || " +
            "execution(* com.example.refactor.service.*Service.query*Users*()) || " +
            "execution(* com.example.refactor.service.*Service.getAll*Users*())")
    public Object aroundUserListQuery(ProceedingJoinPoint joinPoint) throws Throwable {
        if (snapshotService == null) {
            return joinPoint.proceed();
        }
        
        Object result = joinPoint.proceed();
        if (result instanceof List) {
            List<?> users = (List<?>) result;
            if (!users.isEmpty() && users.get(0) instanceof User) {
                return users.stream()
                        .map(user -> snapshotService.getUserWithSnapshot(((User) user).getId()))
                        .filter(user -> user != null)
                        .collect(Collectors.toList());
            }
        }
        return result;
    }
    
    /**
     * 拦截所有更新User的方法（更新操作）
     * 匹配模式：* com.example.refactor.service.*Service.updateUser*(Long, ..)
     */
    @Around("execution(* com.example.refactor.service.*Service.updateUser*(Long, ..)) || " +
            "execution(* com.example.refactor.service.*Service.modifyUser*(Long, ..)) || " +
            "execution(* com.example.refactor.service.*Service.editUser*(Long, ..)) || " +
            "execution(* com.example.refactor.service.*Service.changeUser*(Long, ..))")
    public Object aroundUserUpdate(ProceedingJoinPoint joinPoint) throws Throwable {
        if (snapshotService == null) {
            return joinPoint.proceed();
        }
        
        // 在更新前创建快照
        Object[] args = joinPoint.getArgs();
        if (args.length > 0 && args[0] instanceof Long) {
            Long userId = (Long) args[0];
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                snapshotService.createSnapshot(userId, "用户信息更新");
            }
        }
        
        return joinPoint.proceed();
    }
    
    /**
     * 拦截所有删除User的方法（删除操作）
     * 匹配模式：* com.example.refactor.service.*Service.deleteUser*(Long)
     */
    @Around("execution(* com.example.refactor.service.*Service.deleteUser*(Long)) || " +
            "execution(* com.example.refactor.service.*Service.removeUser*(Long)) || " +
            "execution(* com.example.refactor.service.*Service.destroyUser*(Long))")
    public Object aroundUserDelete(ProceedingJoinPoint joinPoint) throws Throwable {
        if (snapshotService == null) {
            return joinPoint.proceed();
        }
        
        // 在删除前创建快照
        Object[] args = joinPoint.getArgs();
        if (args.length > 0 && args[0] instanceof Long) {
            Long userId = (Long) args[0];
            User currentUser = snapshotService.getUserWithSnapshot(userId);
            if (currentUser != null && currentUser.getSnapshotId() != null) {
                snapshotService.createSnapshot(userId, "用户删除前备份");
            }
        }
        
        return joinPoint.proceed();
    }
    
    /**
     * 拦截所有创建User的方法（创建操作）
     * 匹配模式：* com.example.refactor.service.*Service.createUser*(..)
     */
    @Around("execution(* com.example.refactor.service.*Service.createUser*(..)) || " +
            "execution(* com.example.refactor.service.*Service.addUser*(..)) || " +
            "execution(* com.example.refactor.service.*Service.newUser*(..))")
    public Object aroundUserCreate(ProceedingJoinPoint joinPoint) throws Throwable {
        // 创建操作不需要快照处理，直接执行
        return joinPoint.proceed();
    }
    
    /**
     * 拦截所有操作User的方法（通用拦截器）
     * 这是一个更宽泛的拦截器，用于捕获其他可能遗漏的方法
     */
    @Around("execution(* com.example.refactor.service.*Service.*(..)) && " +
            "args(userId, ..) && " +
            "execution(* *(Long, ..))")
    public Object aroundUserOperation(ProceedingJoinPoint joinPoint) throws Throwable {
        if (snapshotService == null) {
            return joinPoint.proceed();
        }
        
        String methodName = joinPoint.getSignature().getName();
        Object[] args = joinPoint.getArgs();
        
        // 如果是查询操作
        if (methodName.startsWith("get") || methodName.startsWith("find") || methodName.startsWith("query")) {
            Object result = joinPoint.proceed();
            if (result instanceof User) {
                User user = (User) result;
                if (user != null) {
                    return snapshotService.getUserWithSnapshot(user.getId());
                }
            } else if (result instanceof List) {
                List<?> users = (List<?>) result;
                if (!users.isEmpty() && users.get(0) instanceof User) {
                    return users.stream()
                            .map(user -> snapshotService.getUserWithSnapshot(((User) user).getId()))
                            .filter(user -> user != null)
                            .collect(Collectors.toList());
                }
            }
            return result;
        }
        
        // 如果是更新操作
        if (methodName.startsWith("update") || methodName.startsWith("modify") || methodName.startsWith("edit")) {
            if (args.length > 0 && args[0] instanceof Long) {
                Long userId = (Long) args[0];
                User currentUser = snapshotService.getUserWithSnapshot(userId);
                if (currentUser != null && currentUser.getSnapshotId() != null) {
                    snapshotService.createSnapshot(userId, "用户信息更新");
                }
            }
        }
        
        // 如果是删除操作
        if (methodName.startsWith("delete") || methodName.startsWith("remove")) {
            if (args.length > 0 && args[0] instanceof Long) {
                Long userId = (Long) args[0];
                User currentUser = snapshotService.getUserWithSnapshot(userId);
                if (currentUser != null && currentUser.getSnapshotId() != null) {
                    snapshotService.createSnapshot(userId, "用户删除前备份");
                }
            }
        }
        
        return joinPoint.proceed();
    }
} 