package com.example.refactor.config;

import com.example.refactor.entity.User;
import com.example.refactor.service.HRService;
import com.example.refactor.service.UserManagementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

/**
 * 数据初始化器 - 创建演示数据
 */
@Component
public class DataInitializer implements CommandLineRunner {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Value("${app.user.architecture:v1}")
    private String currentArchitecture;
    
    @Override
    public void run(String... args) throws Exception {
        System.out.println("\n=== 重构演示项目启动 ===");
        System.out.println("当前使用架构: " + currentArchitecture);
        System.out.println("架构说明: " + (currentArchitecture.equals("v2") ? 
                "新的分离架构 - 职责清晰，易于维护" : "原始单体架构 - 所有信息混在一个类中"));
        
        initializeData();
        
        System.out.println("\n=== 演示数据创建完成 ===");
        System.out.println("访问 http://localhost:8080/api/users/architecture 查看当前架构信息");
        System.out.println("访问 http://localhost:8080/api/users 查看所有用户");
        System.out.println("访问 http://localhost:8080/h2-console 查看数据库 (JDBC URL: jdbc:h2:mem:testdb)");
        System.out.println("\n=== 切换架构方法 ===");
        System.out.println("1. 修改 application.yml 中的 app.user.architecture 为 v2");
        System.out.println("2. 或使用命令行参数: --app.user.architecture=v2");
        System.out.println("3. 或使用配置文件: --spring.profiles.active=v2");
    }
    
    private void initializeData() {
        try {
            // 创建用户
            User user1 = userManagementService.createUser("张三", "zhangsan@example.com", "13800138001");
            User user2 = userManagementService.createUser("李四", "lisi@example.com", "13800138002");
            User user3 = userManagementService.createUser("王五", "wangwu@example.com", "13800138003");
            
            System.out.println("\n创建了 3 个用户:");
            System.out.println("- " + user1.getName() + " (" + user1.getEmail() + ")");
            System.out.println("- " + user2.getName() + " (" + user2.getEmail() + ")");
            System.out.println("- " + user3.getName() + " (" + user3.getEmail() + ")");
            
            // 设置员工信息
            hrService.updateEmployeeInfo(user1.getId(), "技术部", "高级工程师", 15000.0);
            hrService.updateEmployeeInfo(user2.getId(), "人事部", "HR专员", 8000.0);
            hrService.updateEmployeeInfo(user3.getId(), "技术部", "初级工程师", 10000.0);
            
            System.out.println("\n设置了员工信息:");
            System.out.println("- 张三: 技术部 高级工程师 15000元");
            System.out.println("- 李四: 人事部 HR专员 8000元");
            System.out.println("- 王五: 技术部 初级工程师 10000元");
            
        } catch (Exception e) {
            System.err.println("初始化数据时出错: " + e.getMessage());
            e.printStackTrace();
        }
    }
}