package com.example.refactor.controller;

import com.example.refactor.entity.User;
import com.example.refactor.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import com.example.refactor.entity.UserSnapshot;

/**
 * 用户控制器 - 演示新旧架构的兼容性
 */
@RestController
@RequestMapping("/api/users")
public class UserController {
    
    // 根据配置自动注入对应版本的服务
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private SecurityService securityService;
    
    @Autowired
    private NotificationService notificationService;
    
    @Autowired(required = false)
    private SnapshotService snapshotService;
    
    @Value("${app.user.architecture:v1}")
    private String currentArchitecture;
    
    @GetMapping("/architecture")
    public ResponseEntity<Map<String, String>> getArchitectureInfo() {
        Map<String, String> info = new HashMap<>();
        info.put("current_architecture", currentArchitecture);
        info.put("description", currentArchitecture.equals("v2") ? 
                "使用新的分离架构" : "使用原始单体架构");
        return ResponseEntity.ok(info);
    }
    
    @PostMapping
    public ResponseEntity<User> createUser(@RequestBody CreateUserRequest request) {
        User user = userManagementService.createUser(
                request.getName(), 
                request.getEmail(), 
                request.getPhone()
        );
        return ResponseEntity.ok(user);
    }
    
    @GetMapping("/{id}")
    public ResponseEntity<User> getUser(@PathVariable Long id) {
        User user = userManagementService.getUserById(id);
        if (user != null) {
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    @PutMapping("/{id}")
    public ResponseEntity<User> updateUser(@PathVariable Long id, @RequestBody UpdateUserRequest request) {
        User user = userManagementService.updateUser(
                id, 
                request.getName(), 
                request.getEmail(), 
                request.getPhone()
        );
        if (user != null) {
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteUser(@PathVariable Long id) {
        userManagementService.deleteUser(id);
        return ResponseEntity.ok().build();
    }
    
    @GetMapping
    public ResponseEntity<List<User>> getAllUsers() {
        List<User> users = userManagementService.getAllActiveUsers();
        return ResponseEntity.ok(users);
    }
    
    @PutMapping("/{id}/employee")
    public ResponseEntity<User> updateEmployeeInfo(@PathVariable Long id, @RequestBody EmployeeUpdateRequest request) {
        User user = hrService.updateEmployeeInfo(
                id, 
                request.getDepartment(), 
                request.getPosition(), 
                request.getSalary()
        );
        if (user != null) {
            return ResponseEntity.ok(user);
        }
        return ResponseEntity.notFound().build();
    }
    
    @GetMapping("/department/{department}")
    public ResponseEntity<List<User>> getEmployeesByDepartment(@PathVariable String department) {
        List<User> employees = hrService.getEmployeesByDepartment(department);
        return ResponseEntity.ok(employees);
    }
    
    @PostMapping("/{id}/lock")
    public ResponseEntity<Void> lockUser(@PathVariable Long id) {
        securityService.lockUser(id);
        return ResponseEntity.ok().build();
    }
    
    @PostMapping("/{id}/unlock")
    public ResponseEntity<Void> unlockUser(@PathVariable Long id) {
        securityService.unlockUser(id);
        return ResponseEntity.ok().build();
    }
    
    @PostMapping("/{id}/notify")
    public ResponseEntity<Void> sendNotification(@PathVariable Long id, @RequestBody NotificationRequest request) {
        notificationService.sendEmailNotification(id, request.getMessage());
        return ResponseEntity.ok().build();
    }
    
    // ==================== 快照相关API ====================
    
    /**
     * 为用户创建快照
     */
    @PostMapping("/{id}/snapshots")
    public ResponseEntity<Map<String, String>> createSnapshot(
            @PathVariable Long id, 
            @RequestBody CreateSnapshotRequest request) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().body(Map.of("message", "快照功能未启用"));
        }
        
        try {
            String snapshotId = snapshotService.createSnapshot(id, request.getReason());
            Map<String, String> response = new HashMap<>();
            response.put("snapshotId", snapshotId);
            response.put("message", "快照创建成功");
            return ResponseEntity.ok(response);
        } catch (IllegalArgumentException e) {
            return ResponseEntity.badRequest().build();
        }
    }
    
    /**
     * 为用户创建快照并应用
     */
    @PostMapping("/{id}/snapshots/apply")
    public ResponseEntity<Map<String, String>> createAndApplySnapshot(
            @PathVariable Long id, 
            @RequestBody CreateSnapshotRequest request) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().body(Map.of("message", "快照功能未启用"));
        }
        
        try {
            String snapshotId = snapshotService.createAndApplySnapshot(id, request.getReason());
            Map<String, String> response = new HashMap<>();
            response.put("snapshotId", snapshotId);
            response.put("message", "快照创建并应用成功");
            return ResponseEntity.ok(response);
        } catch (IllegalArgumentException e) {
            return ResponseEntity.badRequest().build();
        }
    }
    
    /**
     * 获取用户的所有快照
     */
    @GetMapping("/{id}/snapshots")
    public ResponseEntity<List<Map<String, Object>>> getUserSnapshots(@PathVariable Long id) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().build();
        }
        
        List<UserSnapshot> snapshots = snapshotService.getUserSnapshots(id);
        List<Map<String, Object>> response = snapshots.stream()
                .map(snapshot -> {
                    Map<String, Object> snapshotInfo = new HashMap<>();
                    snapshotInfo.put("snapshotId", snapshot.getSnapshotId());
                    snapshotInfo.put("createTime", snapshot.getSnapshotCreateTime());
                    snapshotInfo.put("reason", snapshot.getSnapshotReason());
                    return snapshotInfo;
                })
                .collect(Collectors.toList());
        return ResponseEntity.ok(response);
    }
    
    /**
     * 恢复到指定快照
     */
    @PostMapping("/{id}/snapshots/{snapshotId}/restore")
    public ResponseEntity<Map<String, String>> restoreToSnapshot(
            @PathVariable Long id, 
            @PathVariable String snapshotId) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().body(Map.of("message", "快照功能未启用"));
        }
        
        boolean success = snapshotService.restoreToSnapshot(id, snapshotId);
        Map<String, String> response = new HashMap<>();
        if (success) {
            response.put("message", "快照恢复成功");
            return ResponseEntity.ok(response);
        } else {
            response.put("message", "快照恢复失败");
            return ResponseEntity.badRequest().body(response);
        }
    }
    
    /**
     * 清除用户的快照设置
     */
    @DeleteMapping("/{id}/snapshots")
    public ResponseEntity<Map<String, String>> clearSnapshot(@PathVariable Long id) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().body(Map.of("message", "快照功能未启用"));
        }
        
        boolean success = snapshotService.clearSnapshot(id);
        Map<String, String> response = new HashMap<>();
        if (success) {
            response.put("message", "快照设置清除成功");
            return ResponseEntity.ok(response);
        } else {
            response.put("message", "快照设置清除失败");
            return ResponseEntity.badRequest().body(response);
        }
    }
    
    /**
     * 删除快照
     */
    @DeleteMapping("/snapshots/{snapshotId}")
    public ResponseEntity<Map<String, String>> deleteSnapshot(@PathVariable String snapshotId) {
        if (snapshotService == null) {
            return ResponseEntity.badRequest().body(Map.of("message", "快照功能未启用"));
        }
        
        try {
            boolean success = snapshotService.deleteSnapshot(snapshotId);
            Map<String, String> response = new HashMap<>();
            if (success) {
                response.put("message", "快照删除成功");
                return ResponseEntity.ok(response);
            } else {
                response.put("message", "快照删除失败");
                return ResponseEntity.badRequest().body(response);
            }
        } catch (IllegalStateException e) {
            Map<String, String> response = new HashMap<>();
            response.put("message", e.getMessage());
            return ResponseEntity.badRequest().body(response);
        }
    }
    
    // DTO类
    public static class CreateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class UpdateUserRequest {
        private String name;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class EmployeeUpdateRequest {
        private String department;
        private String position;
        private Double salary;
        
        // Getters and Setters
        public String getDepartment() { return department; }
        public void setDepartment(String department) { this.department = department; }
        public String getPosition() { return position; }
        public void setPosition(String position) { this.position = position; }
        public Double getSalary() { return salary; }
        public void setSalary(Double salary) { this.salary = salary; }
    }
    
    public static class NotificationRequest {
        private String message;
        
        // Getters and Setters
        public String getMessage() { return message; }
        public void setMessage(String message) { this.message = message; }
    }
    
    public static class CreateSnapshotRequest {
        private String reason;
        
        // Getters and Setters
        public String getReason() { return reason; }
        public void setReason(String reason) { this.reason = reason; }
    }
}