package com.example.refactor.controller;

import com.example.refactor.adapter.UserVersionAdapter;
import com.example.refactor.dto.UserDTO;
import com.example.refactor.dto.UserMapper;
import com.example.refactor.entity.User;
import com.example.refactor.entity.UserV2;
import com.example.refactor.service.UserManagementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户迁移控制器 - 演示User对象变更的最佳实践
 * 
 * 这个控制器展示了如何在实际应用中处理User实体的变更：
 * 1. 使用DTO模式隔离变更影响
 * 2. 通过适配器处理版本转换
 * 3. 提供渐进式迁移能力
 * 4. 保持API的向后兼容性
 */
@RestController
@RequestMapping("/api/migration")
public class UserMigrationController {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private UserMapper userMapper;
    
    @Autowired
    private UserVersionAdapter versionAdapter;
    
    /**
     * 获取迁移状态信息
     */
    @GetMapping("/status")
    public ResponseEntity<Map<String, Object>> getMigrationStatus() {
        Map<String, Object> status = new HashMap<>();
        
        // 获取所有用户进行转换测试
        List<User> allUsers = userManagementService.getAllActiveUsers();
        UserVersionAdapter.ConversionStats stats = versionAdapter.getConversionStats(allUsers);
        
        status.put("total_users", allUsers.size());
        status.put("conversion_stats", stats);
        status.put("migration_ready", stats.getSuccessRate() > 95.0);
        status.put("recommendations", getMigrationRecommendations(stats));
        
        return ResponseEntity.ok(status);
    }
    
    /**
     * 演示场景1：添加新字段后的API兼容性
     * 即使User实体添加了middleName字段，现有API仍然正常工作
     */
    @PostMapping("/demo/add-field")
    public ResponseEntity<UserDTO> createUserWithNewField(@RequestBody CreateUserWithMiddleNameRequest request) {
        // 创建用户（使用原有逻辑）
        User user = userManagementService.createUser(
            request.getFirstName() + " " + 
            (request.getMiddleName() != null ? request.getMiddleName() + " " : "") + 
            request.getLastName(),
            request.getEmail(),
            request.getPhone()
        );
        
        // 转换为V2以支持新字段
        UserV2 userV2 = versionAdapter.convertToV2(user);
        if (request.getMiddleName() != null) {
            userV2.setMiddleName(request.getMiddleName());
        }
        
        // 转换回V1保存（演示兼容性）
        User updatedUser = versionAdapter.convertToV1(userV2);
        
        // 返回DTO（隐藏内部实现细节）
        UserDTO dto = userMapper.toDTO(updatedUser);
        return ResponseEntity.ok(dto);
    }
    
    /**
     * 演示场景2：字段类型变更的处理
     * salary字段从Double变更为BigDecimal
     */
    @PutMapping("/demo/type-change/{id}/salary")
    public ResponseEntity<UserDTO> updateSalaryWithTypeChange(
            @PathVariable Long id, 
            @RequestBody SalaryUpdateRequest request) {
        
        User user = userManagementService.getUserById(id);
        if (user == null) {
            return ResponseEntity.notFound().build();
        }
        
        // 转换为V2处理新的数据类型
        UserV2 userV2 = versionAdapter.convertToV2(user);
        
        // 使用BigDecimal设置薪资（更精确）
        if (request.getSalaryAmount() != null) {
            userV2.setSalaryAmount(request.getSalaryAmount());
        }
        
        // 转换回V1（自动处理类型转换）
        User updatedUser = versionAdapter.convertToV1(userV2);
        
        // 保存更新
        updatedUser = userManagementService.updateUser(
            id, 
            updatedUser.getName(), 
            updatedUser.getEmail(), 
            updatedUser.getPhone()
        );
        
        UserDTO dto = userMapper.toDTO(updatedUser);
        return ResponseEntity.ok(dto);
    }
    
    /**
     * 演示场景3：删除字段的向后兼容处理
     * fax字段被标记为废弃，但仍然支持
     */
    @PutMapping("/demo/deprecated-field/{id}/fax")
    public ResponseEntity<Map<String, String>> updateDeprecatedField(
            @PathVariable Long id, 
            @RequestBody Map<String, String> request) {
        
        User user = userManagementService.getUserById(id);
        if (user == null) {
            return ResponseEntity.notFound().build();
        }
        
        // 设置废弃字段（仍然支持，但会给出警告）
        String faxValue = request.get("fax");
        if (faxValue != null) {
            user.setFax(faxValue);
        }
        
        Map<String, String> response = new HashMap<>();
        response.put("status", "updated");
        response.put("warning", "fax字段已废弃，建议使用email或phone进行联系");
        response.put("migration_suggestion", "请考虑将传真信息迁移到备注字段或联系方式中");
        
        return ResponseEntity.ok(response);
    }
    
    /**
     * 演示场景4：批量数据迁移
     */
    @PostMapping("/demo/batch-migration")
    public ResponseEntity<Map<String, Object>> performBatchMigration() {
        List<User> allUsers = userManagementService.getAllActiveUsers();
        
        // 转换统计
        UserVersionAdapter.ConversionStats stats = versionAdapter.getConversionStats(allUsers);
        
        // 执行批量转换（演示）
        List<UserV2> convertedUsers = versionAdapter.convertListToV2(allUsers);
        
        // 验证转换结果
        long validConversions = allUsers.stream()
            .mapToLong(user -> {
                UserV2 converted = versionAdapter.convertToV2(user);
                return versionAdapter.validateConversion(user, converted) ? 1 : 0;
            })
            .sum();
        
        Map<String, Object> result = new HashMap<>();
        result.put("original_count", allUsers.size());
        result.put("converted_count", convertedUsers.size());
        result.put("valid_conversions", validConversions);
        result.put("conversion_stats", stats);
        result.put("success_rate", stats.getSuccessRate());
        
        return ResponseEntity.ok(result);
    }
    
    /**
     * 获取用户的不同版本视图
     */
    @GetMapping("/demo/version-comparison/{id}")
    public ResponseEntity<Map<String, Object>> compareUserVersions(@PathVariable Long id) {
        User userV1 = userManagementService.getUserById(id);
        if (userV1 == null) {
            return ResponseEntity.notFound().build();
        }
        
        UserV2 userV2 = versionAdapter.convertToV2(userV1);
        UserDTO dto = userMapper.toDTO(userV1);
        
        Map<String, Object> comparison = new HashMap<>();
        comparison.put("v1_entity", createUserV1Summary(userV1));
        comparison.put("v2_entity", createUserV2Summary(userV2));
        comparison.put("dto_view", dto);
        comparison.put("conversion_valid", versionAdapter.validateConversion(userV1, userV2));
        
        return ResponseEntity.ok(comparison);
    }
    
    /**
     * 获取API变更影响分析
     */
    @GetMapping("/demo/impact-analysis")
    public ResponseEntity<Map<String, Object>> getImpactAnalysis() {
        Map<String, Object> analysis = new HashMap<>();
        
        // 分析各层的影响
        analysis.put("controller_layer", analyzeControllerImpact());
        analysis.put("service_layer", analyzeServiceImpact());
        analysis.put("data_layer", analyzeDataImpact());
        analysis.put("api_compatibility", analyzeAPICompatibility());
        
        return ResponseEntity.ok(analysis);
    }
    
    // 辅助方法
    private List<String> getMigrationRecommendations(UserVersionAdapter.ConversionStats stats) {
        List<String> recommendations = new java.util.ArrayList<>();
        
        if (stats.getSuccessRate() < 95.0) {
            recommendations.add("建议先修复转换失败的数据");
        }
        
        if (stats.getFailed() > 0) {
            recommendations.add("检查失败的转换案例，可能需要数据清理");
        }
        
        recommendations.add("使用DTO模式隔离实体变更影响");
        recommendations.add("通过适配器模式处理版本兼容性");
        recommendations.add("实施渐进式迁移策略");
        
        return recommendations;
    }
    
    private Map<String, Object> createUserV1Summary(User user) {
        Map<String, Object> summary = new HashMap<>();
        summary.put("id", user.getId());
        summary.put("name", user.getName());
        summary.put("email", user.getEmail());
        summary.put("salary_type", "Double");
        summary.put("salary_value", user.getSalary());
        summary.put("has_fax", user.getFax() != null);
        return summary;
    }
    
    private Map<String, Object> createUserV2Summary(UserV2 user) {
        Map<String, Object> summary = new HashMap<>();
        summary.put("id", user.getId());
        summary.put("firstName", user.getFirstName());
        summary.put("middleName", user.getMiddleName());
        summary.put("lastName", user.getLastName());
        summary.put("fullName", user.getName());
        summary.put("email", user.getEmail());
        summary.put("salary_type", "BigDecimal");
        summary.put("salary_value", user.getSalaryAmount());
        summary.put("has_emergency_contact", user.getEmergencyContact() != null);
        summary.put("address_structured", user.getAddressObject() != null);
        return summary;
    }
    
    private Map<String, String> analyzeControllerImpact() {
        Map<String, String> impact = new HashMap<>();
        impact.put("level", "最小");
        impact.put("reason", "使用DTO模式，控制器不直接依赖实体");
        impact.put("required_changes", "仅需更新UserMapper中的映射逻辑");
        return impact;
    }
    
    private Map<String, String> analyzeServiceImpact() {
        Map<String, String> impact = new HashMap<>();
        impact.put("level", "中等");
        impact.put("reason", "服务层直接使用实体，需要适配新字段");
        impact.put("required_changes", "更新业务逻辑以支持新字段，添加兼容性处理");
        return impact;
    }
    
    private Map<String, String> analyzeDataImpact() {
        Map<String, String> impact = new HashMap<>();
        impact.put("level", "高");
        impact.put("reason", "数据库结构变更，需要迁移脚本");
        impact.put("required_changes", "创建迁移脚本，更新Repository接口");
        return impact;
    }
    
    private Map<String, String> analyzeAPICompatibility() {
        Map<String, String> impact = new HashMap<>();
        impact.put("level", "完全兼容");
        impact.put("reason", "API接口保持不变，内部实现透明升级");
        impact.put("benefits", "客户端无需修改，平滑升级");
        return impact;
    }
    
    // 请求DTO类
    public static class CreateUserWithMiddleNameRequest {
        private String firstName;
        private String middleName;
        private String lastName;
        private String email;
        private String phone;
        
        // Getters and Setters
        public String getFirstName() { return firstName; }
        public void setFirstName(String firstName) { this.firstName = firstName; }
        public String getMiddleName() { return middleName; }
        public void setMiddleName(String middleName) { this.middleName = middleName; }
        public String getLastName() { return lastName; }
        public void setLastName(String lastName) { this.lastName = lastName; }
        public String getEmail() { return email; }
        public void setEmail(String email) { this.email = email; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
    }
    
    public static class SalaryUpdateRequest {
        private java.math.BigDecimal salaryAmount;
        
        public java.math.BigDecimal getSalaryAmount() { return salaryAmount; }
        public void setSalaryAmount(java.math.BigDecimal salaryAmount) { this.salaryAmount = salaryAmount; }
    }
}