package com.example.refactor.demo;

import com.example.refactor.entity.User;
import com.example.refactor.service.UserManagementService;
import com.example.refactor.service.HRService;
import com.example.refactor.service.SecurityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

/**
 * 快照功能对比演示
 * 展示不同方案的效果对比
 */
@Component
public class SnapshotComparisonDemo implements CommandLineRunner {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private HRService hrService;
    
    @Autowired
    private SecurityService securityService;
    
    @Override
    public void run(String... args) throws Exception {
        System.out.println("=== 快照功能对比演示 ===");
        
        // 方案1：方法签名模式拦截（完全自动化）
        demonstrateMethodSignatureApproach();
        
        // 方案2：注解方案（声明式）
        demonstrateAnnotationApproach();
        
        // 方案3：手动集成方案（传统方式）
        demonstrateManualIntegrationApproach();
        
        System.out.println("\n=== 演示完成 ===");
    }
    
    /**
     * 演示方案1：方法签名模式拦截
     * 特点：完全自动化，无需修改业务代码
     */
    private void demonstrateMethodSignatureApproach() {
        System.out.println("\n--- 方案1：方法签名模式拦截 ---");
        System.out.println("特点：完全自动化，无需修改业务代码");
        System.out.println("优势：零侵入性，智能识别");
        
        // 创建用户
        User user = userManagementService.createUser("张三", "zhangsan@example.com", "13800138000");
        System.out.println("✓ 创建用户: " + user.getName());
        
        // 查询用户（自动应用快照逻辑）
        User queriedUser = userManagementService.getUserById(user.getId());
        System.out.println("✓ 查询用户: " + queriedUser.getName() + " (自动应用快照)");
        
        // 更新用户（自动创建快照）
        User updatedUser = userManagementService.updateUser(
                user.getId(), 
                "张三（已更新）", 
                "zhangsan.updated@example.com", 
                "13800138001"
        );
        System.out.println("✓ 更新用户: " + updatedUser.getName() + " (自动创建快照)");
        
        // HR操作（自动应用快照）
        User employee = hrService.updateEmployeeInfo(user.getId(), "技术部", "工程师", 8000.0);
        System.out.println("✓ HR更新: " + employee.getDepartment() + " - " + employee.getPosition());
        
        // 安全操作（自动应用快照）
        securityService.lockUser(user.getId());
        User lockedUser = userManagementService.getUserById(user.getId());
        System.out.println("✓ 安全操作: " + lockedUser.getStatus() + " (自动应用快照)");
        
        System.out.println("方案1总结：所有操作都通过方法签名自动识别和拦截");
    }
    
    /**
     * 演示方案2：注解方案
     * 特点：声明式，需要添加注解
     */
    private void demonstrateAnnotationApproach() {
        System.out.println("\n--- 方案2：注解方案 ---");
        System.out.println("特点：声明式，需要添加注解");
        System.out.println("优势：意图明确，可定制化");
        
        // 注意：这里演示的是注解方案的概念
        // 实际代码中，我们使用的是方法签名模式
        System.out.println("示例代码结构：");
        System.out.println("@SnapshotAware(operation = QUERY)");
        System.out.println("public User getUserById(Long id) { ... }");
        System.out.println("");
        System.out.println("@SnapshotAware(operation = UPDATE, createSnapshotBefore = true)");
        System.out.println("public User updateUser(Long id, String name, String email, String phone) { ... }");
        
        System.out.println("方案2总结：通过注解明确声明快照行为");
    }
    
    /**
     * 演示方案3：手动集成方案
     * 特点：传统方式，需要手动修改每个方法
     */
    private void demonstrateManualIntegrationApproach() {
        System.out.println("\n--- 方案3：手动集成方案 ---");
        System.out.println("特点：传统方式，需要手动修改每个方法");
        System.out.println("劣势：代码侵入性强，维护困难");
        
        System.out.println("示例代码结构：");
        System.out.println("public User getUserById(Long id) {");
        System.out.println("    if (snapshotService != null) {");
        System.out.println("        return snapshotService.getUserWithSnapshot(id);");
        System.out.println("    }");
        System.out.println("    return userRepository.findById(id).orElse(null);");
        System.out.println("}");
        System.out.println("");
        System.out.println("public User updateUser(Long id, String name, String email, String phone) {");
        System.out.println("    if (snapshotService != null) {");
        System.out.println("        // 手动创建快照逻辑");
        System.out.println("    }");
        System.out.println("    // 原有业务逻辑");
        System.out.println("}");
        
        System.out.println("方案3总结：需要手动在每个方法中添加快照逻辑");
    }
    
    /**
     * 对比总结
     */
    private void printComparisonSummary() {
        System.out.println("\n=== 方案对比总结 ===");
        System.out.println("方案1（方法签名模式）：");
        System.out.println("  ✓ 完全自动化，零侵入性");
        System.out.println("  ✓ 智能识别，无需修改业务代码");
        System.out.println("  ✓ 支持多种命名规范");
        System.out.println("  ✓ 易于扩展和维护");
        
        System.out.println("\n方案2（注解方案）：");
        System.out.println("  ✓ 意图明确，可定制化");
        System.out.println("  ✓ 声明式编程");
        System.out.println("  - 需要手动添加注解");
        System.out.println("  - 容易遗漏");
        
        System.out.println("\n方案3（手动集成）：");
        System.out.println("  - 代码侵入性强");
        System.out.println("  - 维护困难");
        System.out.println("  - 容易出错");
        System.out.println("  - 扩展性差");
        
        System.out.println("\n推荐：方案1（方法签名模式）");
    }
} 