package com.example.refactor.entity;

import javax.persistence.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * User实体的演进版本 - 演示如何处理实体变更
 * 变更内容：
 * 1. 添加了middleName字段
 * 2. 将salary从Double改为BigDecimal
 * 3. 添加了emergencyContact字段
 * 4. 删除了fax字段（标记为废弃）
 * 5. 重构了地址信息为独立对象
 */
@Entity
@Table(name = "users_v2")
public class UserV2 {
    
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    
    // 基本信息 - 新增middleName
    @Column(nullable = false)
    private String firstName;  // 原name字段拆分
    
    private String middleName;  // 新增字段
    
    @Column(nullable = false)
    private String lastName;   // 原name字段拆分
    
    @Column(unique = true, nullable = false)
    private String email;
    
    private String phone;
    
    // 地址信息重构为嵌入对象
    @Embedded
    private Address address;
    
    // 废弃字段 - 保持向后兼容
    @Deprecated
    @Column(name = "fax")
    private String fax;
    
    // 员工信息
    private String department;
    private String position;
    
    // 薪资字段类型变更：Double -> BigDecimal
    @Deprecated
    @Column(name = "salary_old")
    private Double salary;  // 保留旧字段用于兼容
    
    @Column(name = "salary", precision = 10, scale = 2)
    private BigDecimal salaryAmount;  // 新字段
    
    private LocalDateTime hireDate;
    
    // 新增紧急联系人信息
    @Embedded
    @AttributeOverrides({
        @AttributeOverride(name = "name", column = @Column(name = "emergency_contact_name")),
        @AttributeOverride(name = "phone", column = @Column(name = "emergency_contact_phone")),
        @AttributeOverride(name = "relationship", column = @Column(name = "emergency_contact_relationship"))
    })
    private EmergencyContact emergencyContact;
    
    // 安全和状态信息
    private String status = "ACTIVE";
    private LocalDateTime lastLoginTime;
    private String lastLoginIp;
    private Integer failedLoginAttempts = 0;
    private LocalDateTime lockedUntil;
    private Boolean isLocked = false;
    
    // 审计字段
    @Column(updatable = false)
    private LocalDateTime createdAt;
    private LocalDateTime updatedAt;
    
    // 构造函数
    public UserV2() {
        this.createdAt = LocalDateTime.now();
        this.updatedAt = LocalDateTime.now();
    }
    
    // 兼容性方法 - 处理name字段的拆分
    public String getName() {
        StringBuilder name = new StringBuilder();
        if (firstName != null) name.append(firstName);
        if (middleName != null && !middleName.trim().isEmpty()) {
            if (name.length() > 0) name.append(" ");
            name.append(middleName);
        }
        if (lastName != null) {
            if (name.length() > 0) name.append(" ");
            name.append(lastName);
        }
        return name.toString();
    }
    
    public void setName(String fullName) {
        if (fullName != null) {
            String[] parts = fullName.trim().split("\\s+");
            if (parts.length >= 1) {
                this.firstName = parts[0];
            }
            if (parts.length >= 2) {
                this.lastName = parts[parts.length - 1];
            }
            if (parts.length >= 3) {
                StringBuilder middle = new StringBuilder();
                for (int i = 1; i < parts.length - 1; i++) {
                    if (middle.length() > 0) middle.append(" ");
                    middle.append(parts[i]);
                }
                this.middleName = middle.toString();
            }
        }
    }
    
    // 兼容性方法 - 处理salary类型变更
    public Double getSalary() {
        return salaryAmount != null ? salaryAmount.doubleValue() : salary;
    }
    
    public void setSalary(Double salary) {
        this.salary = salary;
        this.salaryAmount = salary != null ? BigDecimal.valueOf(salary) : null;
    }
    
    // 兼容性方法 - 处理address字段
    public String getAddress() {
        return address != null ? address.getFullAddress() : null;
    }
    
    public void setAddress(String addressStr) {
        if (addressStr != null) {
            this.address = new Address(addressStr);
        }
    }
    
    // 废弃字段的兼容性处理
    @Deprecated
    public String getFax() {
        return fax;
    }
    
    @Deprecated
    public void setFax(String fax) {
        this.fax = fax;
    }
    
    // 标准的Getters and Setters
    public Long getId() { return id; }
    public void setId(Long id) { this.id = id; }
    
    public String getFirstName() { return firstName; }
    public void setFirstName(String firstName) { this.firstName = firstName; }
    
    public String getMiddleName() { return middleName; }
    public void setMiddleName(String middleName) { this.middleName = middleName; }
    
    public String getLastName() { return lastName; }
    public void setLastName(String lastName) { this.lastName = lastName; }
    
    public String getEmail() { return email; }
    public void setEmail(String email) { this.email = email; }
    
    public String getPhone() { return phone; }
    public void setPhone(String phone) { this.phone = phone; }
    
    public Address getAddressObject() { return address; }
    public void setAddressObject(Address address) { this.address = address; }
    
    public String getDepartment() { return department; }
    public void setDepartment(String department) { this.department = department; }
    
    public String getPosition() { return position; }
    public void setPosition(String position) { this.position = position; }
    
    public BigDecimal getSalaryAmount() { return salaryAmount; }
    public void setSalaryAmount(BigDecimal salaryAmount) { 
        this.salaryAmount = salaryAmount;
        this.salary = salaryAmount != null ? salaryAmount.doubleValue() : null;
    }
    
    public LocalDateTime getHireDate() { return hireDate; }
    public void setHireDate(LocalDateTime hireDate) { this.hireDate = hireDate; }
    
    public EmergencyContact getEmergencyContact() { return emergencyContact; }
    public void setEmergencyContact(EmergencyContact emergencyContact) { this.emergencyContact = emergencyContact; }
    
    public String getStatus() { return status; }
    public void setStatus(String status) { this.status = status; }
    
    public LocalDateTime getLastLoginTime() { return lastLoginTime; }
    public void setLastLoginTime(LocalDateTime lastLoginTime) { this.lastLoginTime = lastLoginTime; }
    
    public String getLastLoginIp() { return lastLoginIp; }
    public void setLastLoginIp(String lastLoginIp) { this.lastLoginIp = lastLoginIp; }
    
    public Integer getFailedLoginAttempts() { return failedLoginAttempts; }
    public void setFailedLoginAttempts(Integer failedLoginAttempts) { this.failedLoginAttempts = failedLoginAttempts; }
    
    public LocalDateTime getLockedUntil() { return lockedUntil; }
    public void setLockedUntil(LocalDateTime lockedUntil) { this.lockedUntil = lockedUntil; }
    
    public Boolean getIsLocked() { return isLocked; }
    public void setIsLocked(Boolean isLocked) { this.isLocked = isLocked; }
    
    public LocalDateTime getCreatedAt() { return createdAt; }
    public void setCreatedAt(LocalDateTime createdAt) { this.createdAt = createdAt; }
    
    public LocalDateTime getUpdatedAt() { return updatedAt; }
    public void setUpdatedAt(LocalDateTime updatedAt) { this.updatedAt = updatedAt; }
    
    @PreUpdate
    public void preUpdate() {
        this.updatedAt = LocalDateTime.now();
    }
    
    // 嵌入式地址类
    @Embeddable
    public static class Address {
        private String street;
        private String city;
        private String state;
        private String zipCode;
        private String country;
        
        public Address() {}
        
        public Address(String fullAddress) {
            // 简单解析，实际项目中可能需要更复杂的逻辑
            this.street = fullAddress;
        }
        
        public String getFullAddress() {
            StringBuilder addr = new StringBuilder();
            if (street != null) addr.append(street);
            if (city != null) {
                if (addr.length() > 0) addr.append(", ");
                addr.append(city);
            }
            if (state != null) {
                if (addr.length() > 0) addr.append(", ");
                addr.append(state);
            }
            if (zipCode != null) {
                if (addr.length() > 0) addr.append(" ");
                addr.append(zipCode);
            }
            return addr.toString();
        }
        
        // Getters and Setters
        public String getStreet() { return street; }
        public void setStreet(String street) { this.street = street; }
        public String getCity() { return city; }
        public void setCity(String city) { this.city = city; }
        public String getState() { return state; }
        public void setState(String state) { this.state = state; }
        public String getZipCode() { return zipCode; }
        public void setZipCode(String zipCode) { this.zipCode = zipCode; }
        public String getCountry() { return country; }
        public void setCountry(String country) { this.country = country; }
    }
    
    // 嵌入式紧急联系人类
    @Embeddable
    public static class EmergencyContact {
        private String name;
        private String phone;
        private String relationship;
        
        public EmergencyContact() {}
        
        public EmergencyContact(String name, String phone, String relationship) {
            this.name = name;
            this.phone = phone;
            this.relationship = relationship;
        }
        
        // Getters and Setters
        public String getName() { return name; }
        public void setName(String name) { this.name = name; }
        public String getPhone() { return phone; }
        public void setPhone(String phone) { this.phone = phone; }
        public String getRelationship() { return relationship; }
        public void setRelationship(String relationship) { this.relationship = relationship; }
    }
}