package com.example.refactor.entity.v2;

import javax.persistence.*;
import java.util.Date;

/**
 * 重构后的用户安全信息类
 * 职责：只负责用户的安全和登录相关信息
 */
@Entity
@Table(name = "user_security")
public class UserSecurity {
    @Id
    private Long userProfileId; // 与UserProfile的ID一致
    
    @OneToOne
    @MapsId
    @JoinColumn(name = "user_profile_id")
    private UserProfile userProfile;
    
    @Enumerated(EnumType.STRING)
    private UserStatus status = UserStatus.ACTIVE;
    
    private String lastLoginIp;
    
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastLoginTime;
    
    @Temporal(TemporalType.TIMESTAMP)
    private Date updateTime;
    
    private Integer failedLoginAttempts = 0;
    
    @Temporal(TemporalType.TIMESTAMP)
    private Date lockedUntil;
    
    public enum UserStatus {
        ACTIVE, LOCKED, DELETED, SUSPENDED
    }
    
    public UserSecurity() {}
    
    public UserSecurity(UserProfile userProfile) {
        this.userProfile = userProfile;
        this.status = UserStatus.ACTIVE;
    }
    
    @PreUpdate
    public void preUpdate() {
        this.updateTime = new Date();
    }
    
    public void recordSuccessfulLogin(String clientIp) {
        this.lastLoginIp = clientIp;
        this.lastLoginTime = new Date();
        this.failedLoginAttempts = 0;
        this.lockedUntil = null;
    }
    
    public void recordFailedLogin() {
        this.failedLoginAttempts++;
        if (this.failedLoginAttempts >= 5) {
            this.status = UserStatus.LOCKED;
            // 锁定1小时
            this.lockedUntil = new Date(System.currentTimeMillis() + 3600000);
        }
    }
    
    public boolean isLocked() {
        if (status == UserStatus.LOCKED && lockedUntil != null) {
            if (new Date().after(lockedUntil)) {
                // 自动解锁
                status = UserStatus.ACTIVE;
                lockedUntil = null;
                failedLoginAttempts = 0;
                return false;
            }
            return true;
        }
        return status == UserStatus.LOCKED;
    }
    
    // Getters and Setters
    public Long getUserProfileId() { return userProfileId; }
    public void setUserProfileId(Long userProfileId) { this.userProfileId = userProfileId; }
    
    public UserProfile getUserProfile() { return userProfile; }
    public void setUserProfile(UserProfile userProfile) { this.userProfile = userProfile; }
    
    public UserStatus getStatus() { return status; }
    public void setStatus(UserStatus status) { this.status = status; }
    
    public String getLastLoginIp() { return lastLoginIp; }
    public void setLastLoginIp(String lastLoginIp) { this.lastLoginIp = lastLoginIp; }
    
    public Date getLastLoginTime() { return lastLoginTime; }
    public void setLastLoginTime(Date lastLoginTime) { this.lastLoginTime = lastLoginTime; }
    
    public Date getUpdateTime() { return updateTime; }
    public void setUpdateTime(Date updateTime) { this.updateTime = updateTime; }
    
    public Integer getFailedLoginAttempts() { return failedLoginAttempts; }
    public void setFailedLoginAttempts(Integer failedLoginAttempts) { this.failedLoginAttempts = failedLoginAttempts; }
    
    public Date getLockedUntil() { return lockedUntil; }
    public void setLockedUntil(Date lockedUntil) { this.lockedUntil = lockedUntil; }
}