package com.example.refactor.service;

import com.example.refactor.entity.User;
import com.example.refactor.entity.v2.UserSecurity;
import com.example.refactor.repository.v2.UserSecurityRepository;
import com.example.refactor.service.v2.UserFacadeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 安全服务V2 - 使用新架构的实现
 */
@Service
@Primary
@ConditionalOnProperty(name = "app.user.architecture", havingValue = "v2", matchIfMissing = false)
public class SecurityServiceV2 {
    
    @Autowired
    private UserFacadeService userFacadeService;
    
    @Autowired
    private UserSecurityRepository userSecurityRepository;
    
    public boolean authenticateUser(String email, String password) {
        User user = userFacadeService.getUserByEmail(email);
        if (user != null && "ACTIVE".equals(user.getStatus())) {
            // 简化的认证逻辑
            userFacadeService.recordLogin(user.getId(), getClientIp());
            return true;
        } else if (user != null) {
            // 记录失败登录
            UserSecurity security = userSecurityRepository.findById(user.getId()).orElse(null);
            if (security != null) {
                security.recordFailedLogin();
                userSecurityRepository.save(security);
            }
        }
        return false;
    }
    
    public void updateLoginInfo(Long userId, String clientIp) {
        userFacadeService.recordLogin(userId, clientIp);
    }
    
    public void lockUser(Long userId) {
        userFacadeService.updateUserStatus(userId, "LOCKED");
    }
    
    public void unlockUser(Long userId) {
        userFacadeService.updateUserStatus(userId, "ACTIVE");
    }
    
    public List<User> getRecentlyLoggedInUsers() {
        // 获取最近24小时内登录的用户
        Date since = new Date(System.currentTimeMillis() - 24 * 60 * 60 * 1000);
        List<UserSecurity> recentSecurities = userSecurityRepository.findRecentlyLoggedIn(since);
        
        return recentSecurities.stream()
                .map(security -> userFacadeService.getUserById(security.getUserProfileId()))
                .filter(user -> user != null)
                .collect(Collectors.toList());
    }
    
    public List<User> getLockedUsers() {
        List<UserSecurity> lockedSecurities = userSecurityRepository.findByStatus(UserSecurity.UserStatus.LOCKED);
        
        return lockedSecurities.stream()
                .map(security -> userFacadeService.getUserById(security.getUserProfileId()))
                .filter(user -> user != null)
                .collect(Collectors.toList());
    }
    
    public void unlockExpiredAccounts() {
        List<UserSecurity> expiredLocks = userSecurityRepository.findExpiredLocks();
        for (UserSecurity security : expiredLocks) {
            security.setStatus(UserSecurity.UserStatus.ACTIVE);
            security.setLockedUntil(null);
            security.setFailedLoginAttempts(0);
            userSecurityRepository.save(security);
        }
    }
    
    private String getClientIp() {
        // 模拟获取客户端IP
        return "192.168.1.100";
    }
}