package com.example.refactor.service.v2;

import com.example.refactor.adapter.UserAdapter;
import com.example.refactor.entity.User;
import com.example.refactor.entity.v2.EmployeeInfo;
import com.example.refactor.entity.v2.UserProfile;
import com.example.refactor.entity.v2.UserSecurity;
import com.example.refactor.repository.v2.EmployeeInfoRepository;
import com.example.refactor.repository.v2.UserProfileRepository;
import com.example.refactor.repository.v2.UserSecurityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 用户门面服务 - 新架构的统一入口
 * 职责：提供统一的接口操作分离后的用户相关实体
 */
@Service
@Transactional
public class UserFacadeService {
    
    @Autowired
    private UserProfileRepository userProfileRepository;
    
    @Autowired
    private EmployeeInfoRepository employeeInfoRepository;
    
    @Autowired
    private UserSecurityRepository userSecurityRepository;
    
    @Autowired
    private UserAdapter userAdapter;
    
    /**
     * 创建完整的用户信息
     */
    public User createCompleteUser(String name, String email, String phone, 
                                  String department, String position, Double salary) {
        // 1. 创建用户基础信息
        UserProfile userProfile = new UserProfile(name, email);
        userProfile.setPhone(phone);
        userProfile = userProfileRepository.save(userProfile);
        
        // 2. 创建安全信息
        UserSecurity userSecurity = new UserSecurity(userProfile);
        userSecurity = userSecurityRepository.save(userSecurity);
        
        // 3. 创建员工信息（如果提供了工作相关信息）
        EmployeeInfo employeeInfo = null;
        if (department != null || position != null) {
            employeeInfo = new EmployeeInfo(userProfile, department, position);
            employeeInfo.setSalary(salary);
            employeeInfo = employeeInfoRepository.save(employeeInfo);
        }
        
        // 4. 转换为旧格式返回（向后兼容）
        return userAdapter.toLegacyUser(userProfile, employeeInfo, userSecurity);
    }
    
    /**
     * 更新用户基础信息
     */
    public User updateUserProfile(Long userId, String name, String email, String phone) {
        Optional<UserProfile> profileOpt = userProfileRepository.findById(userId);
        if (profileOpt.isPresent()) {
            UserProfile profile = profileOpt.get();
            profile.setName(name);
            profile.setEmail(email);
            profile.setPhone(phone);
            profile = userProfileRepository.save(profile);
            
            // 获取安全信息
            UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
            EmployeeInfo employeeInfo = employeeInfoRepository.findById(userId).orElse(null);
            
            return userAdapter.toLegacyUser(profile, employeeInfo, security);
        }
        return null;
    }
    
    /**
     * 更新员工信息
     */
    public User updateEmployeeInfo(Long userId, String department, String position, Double salary) {
        UserProfile userProfile = userProfileRepository.findById(userId).orElse(null);
        if (userProfile == null) return null;
        
        EmployeeInfo employeeInfo = employeeInfoRepository.findById(userId).orElse(null);
        if (employeeInfo == null) {
            // 创建新的员工信息
            employeeInfo = new EmployeeInfo(userProfile, department, position);
        } else {
            // 更新现有员工信息
            employeeInfo.setDepartment(department);
            employeeInfo.setPosition(position);
        }
        employeeInfo.setSalary(salary);
        employeeInfo = employeeInfoRepository.save(employeeInfo);
        
        UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
        return userAdapter.toLegacyUser(userProfile, employeeInfo, security);
    }
    
    /**
     * 更新用户安全状态
     */
    public void updateUserStatus(Long userId, String status) {
        UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
        if (security != null) {
            security.setStatus(UserSecurity.UserStatus.valueOf(status));
            userSecurityRepository.save(security);
        }
    }
    
    /**
     * 记录登录信息
     */
    public void recordLogin(Long userId, String clientIp) {
        UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
        if (security != null) {
            security.recordSuccessfulLogin(clientIp);
            userSecurityRepository.save(security);
        }
    }
    
    /**
     * 获取用户完整信息
     */
    public User getUserById(Long userId) {
        UserProfile profile = userProfileRepository.findById(userId).orElse(null);
        if (profile == null) return null;
        
        EmployeeInfo employeeInfo = employeeInfoRepository.findById(userId).orElse(null);
        UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
        
        return userAdapter.toLegacyUser(profile, employeeInfo, security);
    }
    
    /**
     * 根据邮箱获取用户
     */
    public User getUserByEmail(String email) {
        UserProfile profile = userProfileRepository.findByEmail(email).orElse(null);
        if (profile == null) return null;
        
        return getUserById(profile.getId());
    }
    
    /**
     * 获取部门所有员工
     */
    public List<User> getEmployeesByDepartment(String department) {
        List<EmployeeInfo> employees = employeeInfoRepository.findByDepartment(department);
        return employees.stream()
                .map(emp -> {
                    UserSecurity security = userSecurityRepository.findById(emp.getUserProfileId()).orElse(null);
                    return userAdapter.toLegacyUser(emp.getUserProfile(), emp, security);
                })
                .collect(Collectors.toList());
    }
    
    /**
     * 获取活跃用户列表
     */
    public List<User> getActiveUsers() {
        List<UserSecurity> activeSecurities = userSecurityRepository.findByStatus(UserSecurity.UserStatus.ACTIVE);
        return activeSecurities.stream()
                .map(security -> {
                    EmployeeInfo employeeInfo = employeeInfoRepository.findById(security.getUserProfileId()).orElse(null);
                    return userAdapter.toLegacyUser(security.getUserProfile(), employeeInfo, security);
                })
                .collect(Collectors.toList());
    }
    
    /**
     * 软删除用户
     */
    public void deleteUser(Long userId) {
        UserSecurity security = userSecurityRepository.findById(userId).orElse(null);
        if (security != null) {
            security.setStatus(UserSecurity.UserStatus.DELETED);
            userSecurityRepository.save(security);
        }
    }
}