package com.example.refactor.aspect;

import com.example.refactor.entity.User;
import com.example.refactor.service.UserManagementService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.transaction.annotation.Transactional;

import static org.junit.jupiter.api.Assertions.*;

/**
 * 快照切面测试类
 * 验证AOP注解方案是否正常工作
 */
@SpringBootTest
@ActiveProfiles("test")
@Transactional
public class SnapshotAspectTest {
    
    @Autowired
    private UserManagementService userManagementService;
    
    private User testUser;
    
    @BeforeEach
    void setUp() {
        // 创建测试用户
        testUser = userManagementService.createUser("测试用户", "test@example.com", "13800138000");
    }
    
    @Test
    void testGetUserByIdWithAspect() {
        // 测试查询方法是否被切面拦截
        User user = userManagementService.getUserById(testUser.getId());
        
        // 验证返回的是原始数据（因为快照功能未启用）
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertNull(user.getSnapshotId());
    }
    
    @Test
    void testUpdateUserWithAspect() {
        // 测试更新方法是否被切面拦截
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "更新后的用户", 
                "updated@example.com", 
                "13800138001"
        );
        
        // 验证更新成功
        assertNotNull(updatedUser);
        assertEquals("更新后的用户", updatedUser.getName());
        assertEquals("updated@example.com", updatedUser.getEmail());
    }
    
    @Test
    void testDeleteUserWithAspect() {
        // 测试删除方法是否被切面拦截
        userManagementService.deleteUser(testUser.getId());
        
        // 验证删除成功（状态变为DELETED）
        User deletedUser = userManagementService.getUserById(testUser.getId());
        assertNotNull(deletedUser);
        assertEquals("DELETED", deletedUser.getStatus());
    }
    
    @Test
    void testGetAllActiveUsersWithAspect() {
        // 创建多个用户
        userManagementService.createUser("用户1", "user1@example.com", "13800138001");
        userManagementService.createUser("用户2", "user2@example.com", "13800138002");
        
        // 测试列表查询方法是否被切面拦截
        var users = userManagementService.getAllActiveUsers();
        
        // 验证返回的是原始数据
        assertNotNull(users);
        assertTrue(users.size() >= 3); // 至少包含3个用户
        
        // 验证所有用户都没有快照ID
        users.forEach(user -> assertNull(user.getSnapshotId()));
    }
} 