package com.example.refactor.service;

import com.example.refactor.entity.User;
import com.example.refactor.repository.UserRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * 用户管理服务测试类
 * 验证快照功能是否正常工作
 */
@SpringBootTest
@ActiveProfiles("test")
@Transactional
public class UserManagementServiceTest {
    
    @Autowired
    private UserManagementService userManagementService;
    
    @Autowired
    private UserRepository userRepository;
    
    @Autowired(required = false)
    private SnapshotService snapshotService;
    
    private User testUser;
    
    @BeforeEach
    void setUp() {
        // 创建测试用户
        testUser = userManagementService.createUser("测试用户", "test@example.com", "13800138000");
    }
    
    @Test
    void testGetUserByIdWithoutSnapshot() {
        // 获取用户（没有快照）
        User user = userManagementService.getUserById(testUser.getId());
        
        // 验证返回的是原始数据
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertNull(user.getSnapshotId());
    }
    
    @Test
    void testGetUserByIdWithSnapshot() {
        if (snapshotService == null) {
            // 如果快照服务未启用，跳过此测试
            return;
        }
        
        // 创建快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 获取用户（应该返回快照数据）
        User user = userManagementService.getUserById(testUser.getId());
        
        // 验证返回的是快照数据
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertEquals(snapshotId, user.getSnapshotId());
    }
    
    @Test
    void testGetAllActiveUsersWithoutSnapshot() {
        // 创建多个用户
        userManagementService.createUser("用户1", "user1@example.com", "13800138001");
        userManagementService.createUser("用户2", "user2@example.com", "13800138002");
        
        // 获取所有活跃用户
        List<User> users = userManagementService.getAllActiveUsers();
        
        // 验证返回的是原始数据
        assertNotNull(users);
        assertTrue(users.size() >= 3); // 至少包含3个用户
        
        // 验证所有用户都没有快照ID
        users.forEach(user -> assertNull(user.getSnapshotId()));
    }
    
    @Test
    void testGetAllActiveUsersWithSnapshot() {
        if (snapshotService == null) {
            // 如果快照服务未启用，跳过此测试
            return;
        }
        
        // 为第一个用户创建快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 创建另一个用户
        User user2 = userManagementService.createUser("用户2", "user2@example.com", "13800138002");
        
        // 获取所有活跃用户
        List<User> users = userManagementService.getAllActiveUsers();
        
        // 验证返回的数据
        assertNotNull(users);
        assertTrue(users.size() >= 2);
        
        // 找到第一个用户，应该使用快照数据
        User user1 = users.stream()
                .filter(u -> u.getId().equals(testUser.getId()))
                .findFirst()
                .orElse(null);
        
        assertNotNull(user1);
        assertEquals(snapshotId, user1.getSnapshotId());
        
        // 找到第二个用户，应该使用原始数据
        User user2Result = users.stream()
                .filter(u -> u.getId().equals(user2.getId()))
                .findFirst()
                .orElse(null);
        
        assertNotNull(user2Result);
        assertNull(user2Result.getSnapshotId());
    }
    
    @Test
    void testGetUserByEmailWithoutSnapshot() {
        // 根据邮箱获取用户
        User user = userManagementService.getUserByEmail("test@example.com");
        
        // 验证返回的是原始数据
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertNull(user.getSnapshotId());
    }
    
    @Test
    void testGetUserByEmailWithSnapshot() {
        if (snapshotService == null) {
            // 如果快照服务未启用，跳过此测试
            return;
        }
        
        // 创建快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 根据邮箱获取用户
        User user = userManagementService.getUserByEmail("test@example.com");
        
        // 验证返回的是快照数据
        assertNotNull(user);
        assertEquals("测试用户", user.getName());
        assertEquals("test@example.com", user.getEmail());
        assertEquals(snapshotId, user.getSnapshotId());
    }
    
    @Test
    void testUpdateUserWithSnapshot() {
        if (snapshotService == null) {
            // 如果快照服务未启用，跳过此测试
            return;
        }
        
        // 创建快照
        String snapshotId = snapshotService.createAndApplySnapshot(testUser.getId(), "测试快照");
        
        // 更新用户信息
        User updatedUser = userManagementService.updateUser(
                testUser.getId(), 
                "更新后的用户", 
                "updated@example.com", 
                "13800138001"
        );
        
        // 验证更新成功
        assertNotNull(updatedUser);
        assertEquals("更新后的用户", updatedUser.getName());
        assertEquals("updated@example.com", updatedUser.getEmail());
        
        // 验证快照ID被清除（因为更新了原始数据）
        assertNull(updatedUser.getSnapshotId());
    }
} 